@@ -1,4 +1,5 @@
 use crate::auditwheel::PlatformTag;
+use crate::target::Arch;
 use once_cell::sync::Lazy;
 use serde::Deserialize;
 use std::cmp::{Ordering, PartialOrd};
@@ -29,17 +30,20 @@ pub static MUSLLINUX_POLICIES: Lazy<Vec<Policy>> = Lazy::new(|| {
 /// Manylinux policy
 #[derive(Debug, Clone, PartialEq, Deserialize)]
 pub struct Policy {
-    /// manylinux platform tag name
+    /// platform tag name
     pub name: String,
-    /// manylinux platform tag aliases
+    /// platform tag aliases
     pub aliases: Vec<String>,
     /// policy priority. Tags supporting more platforms have higher priority
     pub priority: i64,
     /// platform architecture to symbol versions map
     #[serde(rename = "symbol_versions")]
     pub symbol_versions: HashMap<String, HashMap<String, HashSet<String>>>,
+    /// whitelisted libraries
     #[serde(rename = "lib_whitelist")]
     pub lib_whitelist: HashSet<String>,
+    /// blacklisted symbols of whitelisted libraries
+    pub blacklist: HashMap<String, HashSet<String>>,
 }
 
 impl Default for Policy {
@@ -55,7 +59,7 @@ impl Display for Policy {
             f.write_str(&self.name)
         } else {
             f.write_fmt(format_args!(
-                "{}(aka {})",
+                "{} (aka {})",
                 &self.name,
                 self.aliases.join(",")
             ))
@@ -70,16 +74,6 @@ impl PartialOrd for Policy {
 }
 
 impl Policy {
-    /// Get highest priority policy than self
-    pub fn higher_priority_policies(&self) -> impl Iterator<Item = &Policy> {
-        let policies = if self.name.starts_with("musllinux") {
-            &MUSLLINUX_POLICIES
-        } else {
-            &MANYLINUX_POLICIES
-        };
-        policies.iter().filter(move |p| p.priority > self.priority)
-    }
-
     /// Get platform tag from this policy
     pub fn platform_tag(&self) -> PlatformTag {
         self.name.parse().expect("unknown platform tag")
@@ -97,11 +91,29 @@ impl Policy {
             .find(|p| p.name == name || p.aliases.iter().any(|alias| alias == name))
             .cloned()
     }
+
+    pub(crate) fn fixup_musl_libc_so_name(&mut self, target_arch: Arch) {
+        // Fixup musl libc lib_whitelist
+        if self.name.starts_with("musllinux") && self.lib_whitelist.remove("libc.so") {
+            let new_soname = match target_arch {
+                Arch::Aarch64 => "libc.musl-aarch64.so.1",
+                Arch::Armv7L => "libc.musl-armv7.so.1",
+                Arch::Powerpc64Le => "libc.musl-ppc64le.so.1",
+                Arch::Powerpc64 => "", // musllinux doesn't support ppc64
+                Arch::X86 => "libc.musl-x86.so.1",
+                Arch::X86_64 => "libc.musl-x86_64.so.1",
+                Arch::S390X => "libc.musl-s390x.so.1",
+            };
+            if !new_soname.is_empty() {
+                self.lib_whitelist.insert(new_soname.to_string());
+            }
+        }
+    }
 }
 
 #[cfg(test)]
 mod test {
-    use super::{Policy, MANYLINUX_POLICIES, MUSLLINUX_POLICIES};
+    use super::{Arch, Policy, MANYLINUX_POLICIES, MUSLLINUX_POLICIES};
 
     #[test]
     fn test_load_policy() {
@@ -132,4 +144,11 @@ mod test {
             let _tag = policy.platform_tag();
         }
     }
+
+    #[test]
+    fn test_policy_musllinux_fixup_libc_so_name() {
+        let mut policy = Policy::from_name("musllinux_1_1").unwrap();
+        policy.fixup_musl_libc_so_name(Arch::Aarch64);
+        assert!(policy.lib_whitelist.contains("libc.musl-aarch64.so.1"));
+    }
 }
