@@ -5,18 +5,17 @@
 //!
 //! # Cargo features
 //!
-//! Default features: auditwheel, log, upload, rustls
-//!
-//! - auditwheel: Reimplements the more important part of the auditwheel
-//! package in rust. A wheel is checked by default, unless deactivated by cli arguments
+//! Default features: log, upload, rustls, human-panic
 //!
 //! - log: Configures pretty-env-logger, even though maturin doesn't use logging itself.
 //!
-//! - upload: Uses reqwest to add the upload command.
+//! - upload: Uses ureq to add the upload command.
 //!
-//! - rustls: Makes reqwest use the rustls stack so that we can build maturin in a cent os 5
+//! - rustls: Makes ureq use the rustls stack so that we can build maturin in a CentOS 6
 //! docker container and which maturin itself manylinux compliant.
 //!
+//! - native-tls: Makes ureq use the platform native tls stack
+//!
 //! - human-panic: Adds https://github.com/rust-clique/human-panic
 //!
 //! - password-storage (off by default): Uses the keyring package to store the password. keyring
@@ -26,7 +25,6 @@
 
 #![deny(missing_docs)]
 
-pub use crate::auditwheel::{auditwheel_rs, AuditWheelError};
 pub use crate::build_context::{BridgeModel, BuildContext, BuiltWheelMetadata};
 pub use crate::build_options::BuildOptions;
 pub use crate::cargo_toml::CargoToml;
@@ -36,33 +34,26 @@ pub use crate::metadata::{Metadata21, WheelMetadata};
 pub use crate::module_writer::{
     write_dist_info, ModuleWriter, PathWriter, SDistWriter, WheelWriter,
 };
+pub use crate::new_project::{init_project, new_project, GenerateProjectOptions};
 pub use crate::pyproject_toml::PyProjectToml;
 pub use crate::python_interpreter::PythonInterpreter;
-pub use crate::read_distribution::{
-    get_metadata_for_distribution, get_supported_version_for_distribution,
-};
 pub use crate::target::Target;
-pub use auditwheel::Manylinux;
-pub use source_distribution::source_distribution;
 #[cfg(feature = "upload")]
-pub use {
-    crate::registry::Registry,
-    crate::upload::{upload, UploadError},
-};
+pub use crate::upload::{upload, upload_ui, PublishOpt, Registry, UploadError};
+pub use auditwheel::PlatformTag;
 
 mod auditwheel;
 mod build_context;
 mod build_options;
 mod cargo_toml;
 mod compile;
+mod cross_compile;
 mod develop;
 mod metadata;
 mod module_writer;
+mod new_project;
 mod pyproject_toml;
 mod python_interpreter;
-mod read_distribution;
-#[cfg(feature = "upload")]
-mod registry;
 mod source_distribution;
 mod target;
 #[cfg(feature = "upload")]
