@@ -9,20 +9,21 @@ use flate2::write::GzEncoder;
 use flate2::Compression;
 use fs_err as fs;
 use fs_err::File;
+use ignore::WalkBuilder;
 use sha2::{Digest, Sha256};
 use std::collections::HashMap;
 use std::ffi::OsStr;
-#[cfg(not(target_os = "windows"))]
+#[cfg(target_family = "unix")]
 use std::fs::OpenOptions;
 use std::io;
 use std::io::{Read, Write};
-#[cfg(not(target_os = "windows"))]
+#[cfg(target_family = "unix")]
 use std::os::unix::fs::OpenOptionsExt;
 use std::path::{Path, PathBuf};
 use std::process::{Command, Output};
 use std::str;
 use tempfile::{tempdir, TempDir};
-use walkdir::WalkDir;
+use tracing::debug;
 use zip::{self, ZipWriter};
 
 /// Allows writing the module to a wheel or add it directly to the virtualenv
@@ -45,7 +46,7 @@ pub trait ModuleWriter {
         permissions: u32,
     ) -> Result<()>;
 
-    /// Copies the source file the the target path relative to the module base path
+    /// Copies the source file to the target path relative to the module base path
     fn add_file(&mut self, target: impl AsRef<Path>, source: impl AsRef<Path>) -> Result<()> {
         let read_failed_context = format!("Failed to read {}", source.as_ref().display());
         let mut file = File::open(source.as_ref()).context(read_failed_context.clone())?;
@@ -81,16 +82,13 @@ pub struct PathWriter {
 }
 
 impl PathWriter {
-    /// Creates a [ModuleWriter] that adds the modul to the current virtualenv
+    /// Creates a [ModuleWriter] that adds the module to the current virtualenv
     pub fn venv(target: &Target, venv_dir: &Path, bridge: &BridgeModel) -> Result<Self> {
-        let interpreter = PythonInterpreter::check_executable(
-            target.get_venv_python(&venv_dir),
-            &target,
-            &bridge,
-        )?
-        .ok_or_else(|| {
-            anyhow!("Expected `python` to be a python interpreter inside a virtualenv ಠ_ಠ")
-        })?;
+        let interpreter =
+            PythonInterpreter::check_executable(target.get_venv_python(&venv_dir), target, bridge)?
+                .ok_or_else(|| {
+                    anyhow!("Expected `python` to be a python interpreter inside a virtualenv ಠ_ಠ")
+                })?;
 
         let base_path = target.get_venv_site_package(venv_dir, &interpreter);
 
@@ -168,7 +166,7 @@ impl ModuleWriter for PathWriter {
 
         // We only need to set the executable bit on unix
         let mut file = {
-            #[cfg(not(target_os = "windows"))]
+            #[cfg(target_family = "unix")]
             {
                 OpenOptions::new()
                     .create(true)
@@ -218,7 +216,7 @@ impl ModuleWriter for WheelWriter {
         permissions: u32,
     ) -> Result<()> {
         // The zip standard mandates using unix style paths
-        let target = target.as_ref().to_str().unwrap().replace("\\", "/");
+        let target = target.as_ref().to_str().unwrap().replace('\\', "/");
 
         // Unlike users which can use the develop subcommand, the tests have to go through
         // packing a zip which pip than has to unpack. This makes this 2-3 times faster
@@ -231,7 +229,7 @@ impl ModuleWriter for WheelWriter {
             .unix_permissions(permissions)
             .compression_method(compression_method);
         self.zip.start_file(target.clone(), options)?;
-        self.zip.write_all(&bytes)?;
+        self.zip.write_all(bytes)?;
 
         let hash = base64::encode_config(&Sha256::digest(bytes), base64::URL_SAFE_NO_PAD);
         self.record.push((target, hash, bytes.len()));
@@ -248,7 +246,6 @@ impl WheelWriter {
         tag: &str,
         wheel_dir: &Path,
         metadata21: &Metadata21,
-        scripts: &HashMap<String, String>,
         tags: &[String],
     ) -> Result<WheelWriter> {
         let wheel_path = wheel_dir.join(format!(
@@ -267,11 +264,29 @@ impl WheelWriter {
             wheel_path,
         };
 
-        write_dist_info(&mut builder, &metadata21, &scripts, &tags)?;
+        write_dist_info(&mut builder, metadata21, tags)?;
 
         Ok(builder)
     }
 
+    /// Add a pth file to wheel root for editable installs
+    pub fn add_pth(
+        &mut self,
+        project_layout: &ProjectLayout,
+        metadata21: &Metadata21,
+    ) -> Result<()> {
+        if let Some(python_module) = &project_layout.python_module {
+            let absolute_path = python_module.canonicalize()?;
+            if let Some(python_path) = absolute_path.parent().and_then(|p| p.to_str()) {
+                let name = metadata21.get_distribution_escaped();
+                self.add_bytes(format!("{}.pth", name), python_path.as_bytes())?;
+            } else {
+                println!("⚠️ source code path contains non-Unicode sequences, editable installs may not work.");
+            }
+        }
+        Ok(())
+    }
+
     /// Creates the record file and finishes the zip
     pub fn finish(mut self) -> Result<PathBuf, io::Error> {
         let compression_method = if cfg!(feature = "faster-tests") {
@@ -280,7 +295,7 @@ impl WheelWriter {
             zip::CompressionMethod::Deflated
         };
         let options = zip::write::FileOptions::default().compression_method(compression_method);
-        let record_filename = self.record_file.to_str().unwrap().replace("\\", "/");
+        let record_filename = self.record_file.to_str().unwrap().replace('\\', "/");
         self.zip.start_file(&record_filename, options)?;
         for (filename, hash, len) in self.record {
             self.zip
@@ -327,6 +342,13 @@ impl ModuleWriter for SDistWriter {
     }
 
     fn add_file(&mut self, target: impl AsRef<Path>, source: impl AsRef<Path>) -> Result<()> {
+        if source.as_ref() == self.path {
+            bail!(
+            "Attempting to include the sdist output tarball {} into itself! Check 'cargo package --list' output.",
+            source.as_ref().display()
+            );
+        }
+
         self.tar
             .append_path_with_name(&source, &target)
             .context(format!(
@@ -379,10 +401,13 @@ Root-Is-Purelib: false
 }
 
 /// https://packaging.python.org/specifications/entry-points/
-fn entry_points_txt(entrypoints: &HashMap<String, String, impl std::hash::BuildHasher>) -> String {
+fn entry_points_txt(
+    entry_type: &str,
+    entrypoints: &HashMap<String, String, impl std::hash::BuildHasher>,
+) -> String {
     entrypoints
         .iter()
-        .fold("[console_scripts]\n".to_owned(), |text, (k, v)| {
+        .fold(format!("[{}]\n", entry_type), |text, (k, v)| {
             text + k + "=" + v + "\n"
         })
 }
@@ -413,8 +438,8 @@ where
 
 /// Checks if user has provided their own header at `target/header.h`, otherwise
 /// we run cbindgen to generate one.
-fn cffi_header(crate_dir: &Path, tempdir: &TempDir) -> Result<PathBuf> {
-    let maybe_header = crate_dir.join("target").join("header.h");
+fn cffi_header(crate_dir: &Path, target_dir: &Path, tempdir: &TempDir) -> Result<PathBuf> {
+    let maybe_header = target_dir.join("header.h");
 
     if maybe_header.is_file() {
         println!("💼 Using the existing header at {}", maybe_header.display());
@@ -458,9 +483,13 @@ fn cffi_header(crate_dir: &Path, tempdir: &TempDir) -> Result<PathBuf> {
 /// how to load the shared library without the header and then writes those instructions to a
 /// file called `ffi.py`. This `ffi.py` will expose an object called `ffi`. This object is used
 /// in `__init__.py` to load the shared library into a module called `lib`.
-pub fn generate_cffi_declarations(crate_dir: &Path, python: &Path) -> Result<String> {
+pub fn generate_cffi_declarations(
+    crate_dir: &Path,
+    target_dir: &Path,
+    python: &Path,
+) -> Result<String> {
     let tempdir = tempdir()?;
-    let header = cffi_header(&crate_dir, &tempdir)?;
+    let header = cffi_header(crate_dir, target_dir, &tempdir)?;
 
     let ffi_py = tempdir.as_ref().join("ffi.py");
 
@@ -481,7 +510,7 @@ recompiler.make_py_source(ffi, "ffi", r"{ffi_py}")
         header = header.display(),
     );
 
-    let output = call_python(&python, &["-c", &cffi_invocation])?;
+    let output = call_python(python, &["-c", &cffi_invocation])?;
     let install_cffi = if !output.status.success() {
         // First, check whether the error was cffi not being installed
         let last_line = str::from_utf8(&output.stderr)?.lines().last().unwrap_or("");
@@ -490,7 +519,7 @@ recompiler.make_py_source(ffi, "ffi", r"{ffi_py}")
             // We don't want to modify any global environment
             // https://stackoverflow.com/a/42580137/3549270
             let output = call_python(
-                &python,
+                python,
                 &["-c", "import sys\nprint(sys.base_prefix != sys.prefix)"],
             )?;
 
@@ -499,7 +528,7 @@ recompiler.make_py_source(ffi, "ffi", r"{ffi_py}")
                 "False" => false,
                 _ => {
                     println!(
-                        "⚠  Failed to determine whether python at {:?} is running inside a virtualenv",
+                        "⚠️ Failed to determine whether python at {:?} is running inside a virtualenv",
                         &python
                     );
                     false
@@ -514,13 +543,13 @@ recompiler.make_py_source(ffi, "ffi", r"{ffi_py}")
 
     // If there was success or an error that was not missing cffi, return here
     if !install_cffi {
-        return handle_cffi_call_result(&python, tempdir, &ffi_py, &output);
+        return handle_cffi_call_result(python, tempdir, &ffi_py, &output);
     }
 
-    println!("⚠  cffi not found. Trying to install it");
+    println!("⚠️ cffi not found. Trying to install it");
     // Call pip through python to don't do the wrong thing when python and pip
     // are coming from different environments
-    let output = call_python(&python, &["-m", "pip", "install", "cffi"])?;
+    let output = call_python(python, &["-m", "pip", "install", "cffi"])?;
     if !output.status.success() {
         bail!(
             "Installing cffi with `{:?} -m pip install cffi` failed: {}\n--- Stdout:\n{}\n--- Stderr:\n{}\n---\nPlease install cffi yourself.",
@@ -533,8 +562,8 @@ recompiler.make_py_source(ffi, "ffi", r"{ffi_py}")
     println!("🎁 Installed cffi");
 
     // Try again
-    let output = call_python(&python, &["-c", &cffi_invocation])?;
-    handle_cffi_call_result(&python, tempdir, &ffi_py, &output)
+    let output = call_python(python, &["-c", &cffi_invocation])?;
+    handle_cffi_call_result(python, tempdir, &ffi_py, &output)
 }
 
 /// Extracted into a function because this is needed twice
@@ -554,7 +583,7 @@ fn handle_cffi_call_result(
         );
     } else {
         // Don't swallow warnings
-        std::io::stderr().write_all(&output.stderr)?;
+        io::stderr().write_all(&output.stderr)?;
 
         let ffi_py_content = fs::read_to_string(&ffi_py)?;
         tempdir.close()?;
@@ -563,6 +592,7 @@ fn handle_cffi_call_result(
 }
 
 /// Copies the shared library into the module, which is the only extra file needed with bindings
+#[allow(clippy::too_many_arguments)]
 pub fn write_bindings_module(
     writer: &mut impl ModuleWriter,
     project_layout: &ProjectLayout,
@@ -570,11 +600,11 @@ pub fn write_bindings_module(
     artifact: &Path,
     python_interpreter: Option<&PythonInterpreter>,
     target: &Target,
-    develop: bool,
+    editable: bool,
 ) -> Result<()> {
-    let ext_name = project_layout.extension_name();
+    let ext_name = &project_layout.extension_name;
     let so_filename = match python_interpreter {
-        Some(python_interpreter) => python_interpreter.get_library_name(&ext_name),
+        Some(python_interpreter) => python_interpreter.get_library_name(ext_name),
         // abi3
         None => {
             if target.is_unix() {
@@ -586,82 +616,126 @@ pub fn write_bindings_module(
         }
     };
 
-    match project_layout {
-        ProjectLayout::Mixed {
-            ref python_module,
-            ref rust_module,
-            ..
-        } => {
-            write_python_part(writer, python_module, &module_name)
+    if let Some(python_module) = &project_layout.python_module {
+        if !editable {
+            write_python_part(writer, python_module)
                 .context("Failed to add the python module to the package")?;
+        }
 
-            if develop {
-                let target = rust_module.join(&so_filename);
-                fs::copy(&artifact, &target).context(format!(
-                    "Failed to copy {} to {}",
-                    artifact.display(),
-                    target.display()
-                ))?;
-            }
+        if editable {
+            let target = project_layout.rust_module.join(&so_filename);
+            // Remove existing so file to avoid triggering SIGSEV in running process
+            // See https://github.com/PyO3/maturin/issues/758
+            let _ = fs::remove_file(&target);
+            fs::copy(&artifact, &target).context(format!(
+                "Failed to copy {} to {}",
+                artifact.display(),
+                target.display()
+            ))?;
+        }
 
-            let relative = rust_module.strip_prefix(python_module.parent().unwrap())?;
+        if !editable {
+            let relative = project_layout
+                .rust_module
+                .strip_prefix(python_module.parent().unwrap())?;
             writer.add_file_with_permissions(relative.join(&so_filename), &artifact, 0o755)?;
         }
-        ProjectLayout::PureRust(_) => {
-            writer.add_file_with_permissions(so_filename, &artifact, 0o755)?;
+    } else {
+        let module = PathBuf::from(module_name);
+        writer.add_directory(&module)?;
+        // Reexport the shared library as if it were the top level module
+        writer.add_bytes(
+            &module.join("__init__.py"),
+            format!(
+                r#"from .{module_name} import *
+
+__doc__ = {module_name}.__doc__
+if hasattr({module_name}, "__all__"):
+    __all__ = {module_name}.__all__"#,
+                module_name = module_name
+            )
+            .as_bytes(),
+        )?;
+        let type_stub = project_layout
+            .rust_module
+            .join(format!("{}.pyi", module_name));
+        if type_stub.exists() {
+            writer.add_bytes(
+                &module.join("__init__.pyi"),
+                &fs_err::read(type_stub).context("Failed to read type stub file")?,
+            )?;
+            writer.add_bytes(&module.join("py.typed"), b"")?;
+            println!("📖 Found type stub file at {}.pyi", module_name);
         }
+        writer.add_file_with_permissions(&module.join(so_filename), &artifact, 0o755)?;
     }
 
     Ok(())
 }
 
 /// Creates the cffi module with the shared library, the cffi declarations and the cffi loader
+#[allow(clippy::too_many_arguments)]
 pub fn write_cffi_module(
     writer: &mut impl ModuleWriter,
     project_layout: &ProjectLayout,
     crate_dir: &Path,
+    target_dir: &Path,
     module_name: &str,
     artifact: &Path,
     python: &Path,
-    develop: bool,
+    editable: bool,
 ) -> Result<()> {
-    let cffi_declarations = generate_cffi_declarations(&crate_dir, python)?;
+    let cffi_declarations = generate_cffi_declarations(crate_dir, target_dir, python)?;
 
     let module;
 
-    match project_layout {
-        ProjectLayout::Mixed {
-            ref python_module,
-            ref rust_module,
-            ref extension_name,
-        } => {
-            write_python_part(writer, python_module, &module_name)
+    if let Some(python_module) = &project_layout.python_module {
+        if !editable {
+            write_python_part(writer, python_module)
                 .context("Failed to add the python module to the package")?;
-
-            if develop {
-                let base_path = python_module.join(&module_name);
-                fs::create_dir_all(&base_path)?;
-                let target = base_path.join("native.so");
-                fs::copy(&artifact, &target).context(format!(
-                    "Failed to copy {} to {}",
-                    artifact.display(),
-                    target.display()
-                ))?;
-                File::create(base_path.join("__init__.py"))?
-                    .write_all(cffi_init_file().as_bytes())?;
-                File::create(base_path.join("ffi.py"))?.write_all(cffi_declarations.as_bytes())?;
-            }
-
-            let relative = rust_module.strip_prefix(python_module.parent().unwrap())?;
-            module = relative.join(extension_name);
         }
-        ProjectLayout::PureRust(_) => module = PathBuf::from(module_name),
+
+        if editable {
+            let base_path = python_module.join(&module_name);
+            fs::create_dir_all(&base_path)?;
+            let target = base_path.join("native.so");
+            fs::copy(&artifact, &target).context(format!(
+                "Failed to copy {} to {}",
+                artifact.display(),
+                target.display()
+            ))?;
+            File::create(base_path.join("__init__.py"))?.write_all(cffi_init_file().as_bytes())?;
+            File::create(base_path.join("ffi.py"))?.write_all(cffi_declarations.as_bytes())?;
+        }
+
+        let relative = project_layout
+            .rust_module
+            .strip_prefix(python_module.parent().unwrap())?;
+        module = relative.join(&project_layout.extension_name);
+        if !editable {
+            writer.add_directory(&module)?;
+        }
+    } else {
+        module = PathBuf::from(module_name);
+        writer.add_directory(&module)?;
+        let type_stub = project_layout
+            .rust_module
+            .join(format!("{}.pyi", module_name));
+        if type_stub.exists() {
+            writer.add_bytes(
+                &module.join("__init__.pyi"),
+                &fs_err::read(type_stub).context("Failed to read type stub file")?,
+            )?;
+            writer.add_bytes(&module.join("py.typed"), b"")?;
+            println!("📖 Found type stub file at {}.pyi", module_name);
+        }
     };
 
-    writer.add_directory(&module)?;
-    writer.add_bytes(&module.join("__init__.py"), cffi_init_file().as_bytes())?;
-    writer.add_bytes(&module.join("ffi.py"), cffi_declarations.as_bytes())?;
-    writer.add_file_with_permissions(&module.join("native.so"), &artifact, 0o755)?;
+    if !editable || project_layout.python_module.is_none() {
+        writer.add_bytes(&module.join("__init__.py"), cffi_init_file().as_bytes())?;
+        writer.add_bytes(&module.join("ffi.py"), cffi_declarations.as_bytes())?;
+        writer.add_file_with_permissions(&module.join("native.so"), &artifact, 0o755)?;
+    }
 
     Ok(())
 }
@@ -688,22 +762,13 @@ pub fn write_bin(
 }
 
 /// Adds the python part of a mixed project to the writer,
-/// excluding older versions of the native library or generated cffi declarations
 pub fn write_python_part(
     writer: &mut impl ModuleWriter,
     python_module: impl AsRef<Path>,
-    module_name: impl AsRef<Path>,
 ) -> Result<()> {
-    for absolute in WalkDir::new(&python_module) {
+    for absolute in WalkBuilder::new(&python_module).hidden(false).build() {
         let absolute = absolute?.into_path();
-
         let relative = absolute.strip_prefix(python_module.as_ref().parent().unwrap())?;
-
-        // Ignore the cffi folder from develop, if any
-        if relative.starts_with(module_name.as_ref().join(&module_name)) {
-            continue;
-        }
-
         if absolute.is_dir() {
             writer.add_directory(relative)?;
         } else {
@@ -726,7 +791,6 @@ pub fn write_python_part(
 pub fn write_dist_info(
     writer: &mut impl ModuleWriter,
     metadata21: &Metadata21,
-    scripts: &HashMap<String, String, impl std::hash::BuildHasher>,
     tags: &[String],
 ) -> Result<()> {
     let dist_info_dir = metadata21.get_dist_info_dir();
@@ -740,12 +804,85 @@ pub fn write_dist_info(
 
     writer.add_bytes(&dist_info_dir.join("WHEEL"), wheel_file(tags).as_bytes())?;
 
-    if !scripts.is_empty() {
+    let mut entry_points = String::new();
+    if !metadata21.scripts.is_empty() {
+        entry_points.push_str(&entry_points_txt("console_scripts", &metadata21.scripts));
+    }
+    if !metadata21.gui_scripts.is_empty() {
+        entry_points.push_str(&entry_points_txt("gui_scripts", &metadata21.gui_scripts));
+    }
+    for (entry_type, scripts) in &metadata21.entry_points {
+        entry_points.push_str(&entry_points_txt(entry_type, scripts));
+    }
+    if !entry_points.is_empty() {
         writer.add_bytes(
             &dist_info_dir.join("entry_points.txt"),
-            entry_points_txt(scripts).as_bytes(),
+            entry_points.as_bytes(),
         )?;
     }
 
+    if !metadata21.license_files.is_empty() {
+        let license_files_dir = dist_info_dir.join("license_files");
+        writer.add_directory(&license_files_dir)?;
+        for path in &metadata21.license_files {
+            let filename = path.file_name().with_context(|| {
+                format!("missing file name for license file {}", path.display())
+            })?;
+            writer.add_file(license_files_dir.join(filename), path)?;
+        }
+    }
+
+    Ok(())
+}
+
+/// If any, copies the data files from the data directory, resolving symlinks to their source.
+/// We resolve symlinks since we require this rather rigid structure while people might need
+/// to save or generate the data in other places
+///
+/// See https://peps.python.org/pep-0427/#file-contents
+pub fn add_data(writer: &mut impl ModuleWriter, data: Option<&Path>) -> Result<()> {
+    let possible_data_dir_names = ["data", "scripts", "headers", "purelib", "platlib"];
+    if let Some(data) = data {
+        for subdir in fs::read_dir(data).context("Failed to read data dir")? {
+            let subdir = subdir?;
+            let dir_name = subdir
+                .file_name()
+                .to_str()
+                .context("Invalid data dir name")?
+                .to_string();
+            if !subdir.path().is_dir() || !possible_data_dir_names.contains(&dir_name.as_str()) {
+                bail!(
+                    "Invalid data dir entry {}. Possible are directories named {}",
+                    subdir.path().display(),
+                    possible_data_dir_names.join(", ")
+                );
+            }
+            debug!("Adding data from {}", subdir.path().display());
+            (|| {
+                for file in WalkBuilder::new(subdir.path())
+                    .standard_filters(false)
+                    .build()
+                {
+                    let file = file?;
+                    let relative = file.path().strip_prefix(data.parent().unwrap())?;
+
+                    if file.path_is_symlink() {
+                        // Copy the actual file contents, not the link, so that you can create a
+                        // data directory by joining different data sources
+                        let source = fs::read_link(file.path())?;
+                        writer.add_file(relative, source.parent().unwrap())?;
+                    } else if file.path().is_file() {
+                        writer.add_file(relative, file.path())?;
+                    } else if file.path().is_dir() {
+                        writer.add_directory(relative)?;
+                    } else {
+                        bail!("Can't handle data dir entry {}", file.path().display());
+                    }
+                }
+                Ok(())
+            })()
+            .with_context(|| format!("Failed to include data from {}", data.display()))?
+        }
+    }
     Ok(())
 }
